
# MoreDateTags.pl
# Movable Type plugin tags for date-based output
# by Kevin Shay
# http://www.staggernation.com/mtplugins/
# last modified July 12, 2004

use strict;
package MT::Plugin::DateTagsB;

use vars qw( $static_hols $moving_hols $stones $VERSION );
$VERSION = '2.3';

use MT;
use MT::Template::Context;

eval{ require MT::Plugin };
unless ($@) {
    my $plugin = {
        name => "MoreDateTags $VERSION",
        description => 'Additional tags from the DateTags plugin package.',
        doc_link => 'http://www.staggernation.com/mtplugins/DateTags'
    }; 
    MT->add_plugin(new MT::Plugin($plugin));
}

MT::Template::Context->add_tag('MoonPhase' => sub{&_hdlr_moon_phase;});
MT::Template::Context->add_tag('HolidaysUS' => sub{&_hdlr_holidays_us;});
MT::Template::Context->add_tag('AstroSign' => sub{&_hdlr_astro_sign;});
MT::Template::Context->add_tag('Birthstone' => sub{&_hdlr_birthstone;});

use DateTags;
	# keep track of whether we've loaded Date::Calc
my $datecalc = 0;
	# these structures will be populated the first time they're needed
$static_hols = {};
$moving_hols = {};
$stones = [];

sub dt_load_datecalc {
# instead of use()ing Date::Calc, load it the first time it's needed
# (so it'll only load if one of the tags is actually called)
	require Date::Calc;
	import Date::Calc qw( Add_Delta_Days Add_Delta_DHMS Day_of_Year Delta_Days 
		Delta_DHMS Easter_Sunday );
	return 1;
}

sub _hdlr_moon_phase {
	my ($ctx, $args) = @_;
	$datecalc ||= dt_load_datecalc();
	my $ts = dt_get_ts($ctx, $args);
		# seconds in a lunar cycle
	my $period_s = 29.53058867 * 24 * 60 * 60;
	my @flds = qw(y m d h min s);
	my %now = ();
	@now{@flds} = unpack('A4A2A2A2A2A2', $ts);
		# our base date is GMT, so we'll adjust it by the user's timezone
		# (ignoring DST--the calculation is approximate anyway)
	my $timezone = $ctx->stash('blog')->server_offset || 0;
	my %base = ();
		# base date/time: new moon on July 10, 2002, 10:27 AM GMT
	@base{@flds} = Add_Delta_DHMS(2002, 7, 10, 10, 27, 0, 0, $timezone, 0, 0);
		# calculate seconds between base and now
	my ($d_d, $d_h, $d_min, $d_s) = Delta_DHMS(@base{@flds}, @now{@flds});
	my $delta_s = abs(($d_d * 24 * 60 * 60) + ($d_h * 60 * 60) + ($d_min * 60) + $d_s);
		# how far through a lunar cycle are we?
	my $phase_pct = (dt_remainder($delta_s, $period_s) / $period_s) * 100;
	my $phase;
	for ($phase_pct) {
		($phase = 'Waxing Crescent'), last if (($_ >= 2.5) && ($_ < 22.5));
		($phase = 'First Quarter'), last if (($_ >= 22.5) && ($_ < 27.5));
		($phase = 'Waxing Gibbous'), last if (($_ >= 27.5) && ($_ < 47.5));
		($phase = 'Full'), last if (($_ >= 47.5) && ($_ < 52.49));
		($phase = 'Waning Gibbous'), last if (($_ >= 52.5) && ($_ < 73.5));
		($phase = 'Last Quarter'), last if (($_ >= 74.5) && ($_ < 77.5));
		($phase = 'Waning Crescent'), last if (($_ >= 77.5) && ($_ < 97.5));
		$phase = 'New';
	}
	return $phase;
}

sub _hdlr_holidays_us {
	my ($ctx, $args) = @_;
	$datecalc ||= dt_load_datecalc();
	my $ts = dt_get_ts($ctx, $args);
	my @today_hols;
	my ($y, $m, $d) = unpack('A4A2A2', $ts);
	if (!%$static_hols) {
			# populate the static holidays
		$static_hols = dt_static_hols();
	}
	if (defined($static_hols->{"$m$d"})) {
		push(@today_hols, @{$static_hols->{"$m$d"}});
	}
	if (!defined($moving_hols->{$y})) {
			# populate the moving holidays for the current year
		$moving_hols->{$y} = dt_moving_hols($y);
	}
	if (defined($moving_hols->{$y}->{"$m$d"})) {
		push(@today_hols, @{$moving_hols->{$y}->{"$m$d"}});
	}
	my $glue = defined($args->{'glue'}) ? $args->{'glue'} : '; ';
	return join($glue, @today_hols);	
}

sub _hdlr_astro_sign {
	my ($ctx, $args) = @_;
	$datecalc ||= dt_load_datecalc();
	my $ts = dt_get_ts($ctx, $args);
	my ($y, $m, $d) = unpack('A4A2A2', $ts);
	my $doy = Day_of_Year($y, $m, $d);
	dt_doy_within($doy, $y, 1, 1, 1, 19) && return 'Capricorn';
	dt_doy_within($doy, $y, 1, 20, 2, 18) && return 'Aquarius';
	dt_doy_within($doy, $y, 2, 19, 3, 20) && return 'Pisces';
	dt_doy_within($doy, $y, 3, 21, 4, 19) && return 'Aries';
	dt_doy_within($doy, $y, 4, 20, 5, 20) && return 'Taurus';
	dt_doy_within($doy, $y, 5, 21, 6, 21) && return 'Gemini';
	dt_doy_within($doy, $y, 6, 22, 7, 22) && return 'Cancer';
	dt_doy_within($doy, $y, 7, 23, 8, 22) && return 'Leo';
	dt_doy_within($doy, $y, 8, 23, 9, 22) && return 'Virgo';
	dt_doy_within($doy, $y, 9, 23, 10, 22) && return 'Libra';
	dt_doy_within($doy, $y, 10, 23, 11, 21) && return 'Scorpio';
	dt_doy_within($doy, $y, 11, 22, 12, 21) && return 'Sagittarius';
	dt_doy_within($doy, $y, 12, 22, 12, 31) && return 'Capricorn';
	return '';
}

sub _hdlr_birthstone {
	my ($ctx, $args) = @_;
	my $ts = dt_get_ts($ctx, $args);
	my $m = substr($ts, 4, 2);
	unless (@$stones) {
		@$stones = ('Garnet', 'Amethyst', 'Aquamarine', 'Diamond', 
			'Emerald', 'Pearl, Moonstone', 'Ruby', 'Peridot', 
			'Sapphire', 'Opal, Tourmaline', 'Yellow Topaz, Citrine', 
			'Blue Topaz, Turquoise');
	}
		# account for 1-based month value
	return $stones->[$m-1];
}

sub dt_static_hols {
# populate a data structure containing the dates of various static holidays: 
# those that occur on the same date each year.
# return a hash where key = MMDD and value = ref to array of holidays
# (in case we ever end up with multiple holidays on same date)
	return {
		'0101' => ["New Year's Day"],
		'0202' => ["Groundhog Day"],
		'0212' => ["Lincoln's Birthday"],
		'0214' => ["Valentine's Day"],
		'0222' => ["Washington's Birthday"],
		'0317' => ["St. Patrick's Day"],
		'0401' => ["April Fool's Day"],
		'0415' => ["Tax Day"],
		'0614' => ["Flag Day"],
		'0704' => ["Independence Day"],
		'1024' => ["United Nations Day"],
		'1031' => ["Halloween"],
		'1111' => ["Veterans Day"],
		'1224' => ["Christmas Eve"],
		'1225' => ["Christmas Day"],
		'1231' => ["New Year's Eve"]
	};
}

sub dt_moving_hols {
# populate a data structure containing the dates of various moving holidays: 
# those that do not occur on the same date each year.
	my ($y) = @_;
	my %hols = ();
		# loop through our list of holidays, each in the format
		# [N, day, month, name]
		# where N means "Nth [weekday] of the month" (-1 = "last");
		# day = day of the week, starting with Monday = 1
		# e.g. [3, 6, 5, "Armed Forces Day"] is the 3rd Saturday in May
	for my $hol (
		[3, 1, 1, "Martin Luther King Day"],
		[3, 1, 2, "Presidents' Day"],
		[1, 7, 4, "Daylight Savings Time begins"],
		[3, 6, 5, "Armed Forces Day"],
		[2, 7, 5, "Mother's Day"],
		[-1, 1, 5, "Memorial Day"],
		[3, 7, 6, "Father's Day"],
		[1, 1, 9, "Labor Day"],
		[2, 1, 10, 'Columbus Day'],
		[-1, 7, 10, "Daylight Savings Time ends"],
		[4, 4, 11, "Thanksgiving"]
	) {
		my $d = sprintf("%02d", dt_nwomy(@{$hol}[0..2], $y));
		next if ($d == 0);
		my $m = sprintf("%02d", $hol->[2]);
		if (!defined($hols{"$m$d"})) {
			$hols{"$m$d"} = [];
		}
		push(@{$hols{"$m$d"}}, $hol->[3]);
	}
	my %easter;
	@easter{qw(y m d)} = Easter_Sunday($y);
		# holidays based on the date of Easter Sunday
		# (there are lots more, but let's not get too ecclesiastical)
	for my $hol (
		[0, "Easter Sunday"],
		[-47, "Mardi Gras"],
		[-46, "Ash Wednesday"],
		[-7, "Palm Sunday"]
	) {
		my (undef, $m, $d) = Add_Delta_Days(@easter{qw(y m d)}, $hol->[0]);
		for ($m, $d) {
			$_ = sprintf("%02d", $_);
		}
		if (!defined($hols{"$m$d"})) {
			$hols{"$m$d"} = [];
		}
		push(@{$hols{"$m$d"}}, $hol->[1]);
	}
	
	return \%hols;
}


sub dt_doy_within {
# passed a day of the year, a year, and two month/date combinations, test 
# whether the day lies between those two dates (for the same year)
	my ($doy, $y, $m1, $d1, $m2, $d2) = @_;
	return ((Day_of_Year($y, $m1, $d1) <= $doy) && 
			(Day_of_Year($y, $m2, $d2) >= $doy));
}

sub dt_remainder {
# return remainder of division of two numbers (not necessarily integers)
    my ($num, $denom) = @_;
    return $num - ($denom * int($num/$denom));
}

1;
